(**********************************************
	COMS W4115 PLT - 2014 Spring Term Project
	HL-HDL compiler: ast.ml
		Name:	Woon Lee
		UNI:	wgl2107
***********************************************)

type op = And | Or | Equal | Nequal

type v_type = Const | Signal | Insig | Outsig | IOsig

(* type f_type = Insig | Outsig | IOsig *)

type btin_f = Inv | Buf | Lat | Mux

type fun_type = User | Main

type formal = { 
	formal_type : v_type; 
	formal_name : string;
	formal_range : int * int
	}

type expr =							(* expressions *)
	  Lit_int of int				(* 10 *)
	| Lit_hex of string				(* x1010 *)
	| Lit_bin of string				(* b1010 *)
	| Id of string					(* foo *)
	| Binop of expr * op * expr 	(* a AND b *)
	| Single of string * int		(* a[2] *)
	| Range of string * int * int	(* a[3:5] *)
	| Braces of expr				(* (a OR b) *)
	| Assign of expr * expr			(* foo = x10ac *)
	| Call of string * string * expr list	(* foo(a, b) *)
	| Bcall of string * btin_f * expr list	(* inv(a) *)
	
type stmt =						(* statements *)
	  Block of stmt list		(* {....} *)
	| Expr of expr				(* c = a AND b *)
	| If of string * expr * stmt * stmt	(* if (a == b) then {} else {} *)
	| StError of expr			(* missing ; *)

type var_decl = {				(* variable declaration *)
	var_type : v_type;			(* variable type *)
	var_name : string;			(* variable name *)
	var_range : int * int;		(* variable range for bus *)
	var_value : string			(* variable init value *)
	}

type func_decl = {				(* function declaration *)
	func_type : fun_type;		(* user-defined or main *)
	func_name : string;			(* name of the function *)
	formals : formal list;		(* formal arguments *)
	locals : var_decl list;		(* local variables *)
	body : stmt list			(* function body *)
	}

type program = var_decl list * func_decl list	(* global variables, functions *)


(* helper functions for pretty printer *)

let rec string_of_expr = function
	  Lit_int(literal)			-> string_of_int literal
	| Lit_hex(hex_lit)			-> hex_lit
	| Lit_bin(bin_lit)			-> bin_lit
	| Id(id) 					-> id
	| Single(id, index)			-> id ^ "(" ^ (string_of_int index) ^ ")"
	| Range(id, first, last)	-> id ^ "(" ^ (string_of_int first) ^ ":" ^ (string_of_int last) ^ ")"
	| Braces(e)					-> "(" ^ (string_of_expr e) ^ ")"
	| Binop(e1, oper, e2)		-> (string_of_expr e1) ^ " " ^ 
								   (match oper with
										  And	-> "AND "
										| Or	-> "OR "
										| Equal	-> "== "
										| Nequal	-> "!= ") ^ (string_of_expr e2)
	| Assign(e1, e2)			-> (string_of_expr e1) ^ " = " ^ (string_of_expr e2)
	| Call(inst_id, f_id, e_list)		-> inst_id ^ ":" ^ f_id ^ "(" ^ String.concat ", " (List.map string_of_expr e_list) ^ ")" 
	| Bcall(inst_id, btif_id, e_list)	-> 
									(match btif_id with
										  Inv	-> "INV"
										| Buf	-> "BUF"
										| Lat	-> inst_id ^ ":" ^"LAT"
										| Mux	-> inst_id ^ ":" ^"MUX" )
									^ "(" ^ String.concat ", " (List.map string_of_expr e_list) ^ ")" 
	
let rec string_of_stmt = function
	  Block(st)				-> "\t{\n\t" ^ String.concat "\t " (List.map string_of_stmt st) ^ "\t}\n"
	| Expr(e) 				-> "\t" ^ string_of_expr e ^ ";\n"
	| If(pid, e, s, Block([]))	-> "\t"^pid^": if (" ^ (string_of_expr e) ^ ") \tthen\n" ^ string_of_stmt s
	| If(pid, e, s, s1)			-> "\t"^pid^": if (" ^ (string_of_expr e) ^ ") \tthen\n" ^ string_of_stmt s ^
								"\n \telse\n" ^ string_of_stmt s1
	| StError (e)			-> "\t" ^ string_of_expr e ^ ";\n"

let string_of_v_type vtype = match vtype with Const -> "constant" | Signal -> "signal"
											| Insig -> "in" | Outsig -> "out" | IOsig -> "inout"

let string_of_btif btif = match btif with Inv -> "Inv" | Buf -> "Buf" | Lat -> "Lat" | Mux -> "Mux"

(* let string_of_f_type ftype = match ftype with Insig -> "in " | Outsig -> "out " | IOsig -> "inout " *)

let string_of_funtype funtype = match funtype with User -> "User " | Main -> "Main "

let string_of_formal fn_param = string_of_v_type fn_param.formal_type ^ " " ^ fn_param.formal_name ^
	(match (fst fn_param.formal_range) with 
		-1	-> ""
		| _	-> "(" ^ string_of_int (fst fn_param.formal_range) ^ ":" ^ string_of_int (snd fn_param.formal_range) ^ ")")

let string_of_var_decl variable =
	(string_of_v_type variable.var_type) ^ " " ^ variable.var_name ^
	(match (fst variable.var_range) with 
		-1	-> ""
		| _	-> "(" ^ string_of_int (fst variable.var_range) ^ ":" ^ string_of_int (snd variable.var_range) ^ ")")
	^ (match (variable.var_value) with
		"none"	-> ""
		| _		-> " = " ^ variable.var_value) ^ ";\n"
	
let string_of_func_decl func =
	string_of_funtype func.func_type ^ " " ^ func.func_name ^ "(" ^ String.concat ", " (List.map string_of_formal func.formals) ^ 
	")\n{\n" ^ "\t" ^ String.concat "\t" (List.map string_of_var_decl func.locals) ^ "\n" ^ String.concat "" (List.map string_of_stmt func.body) ^ "}\n"	
	  

let string_of_program (global_variables, func_declaraions) =
   String.concat "" (List.map string_of_var_decl (List.rev global_variables)) ^ "\n" ^
   String.concat "\n" (List.map string_of_func_decl func_declaraions)