(**********************************************
	COMS W4115 PLT - 2014 Spring Term Project
	HL-HDL compiler: hlhdl.ml
		Name:	Woon Lee
		UNI:	wgl2107
***********************************************)

open Printf

type action = Ast | Compile | TestBench | Error

let out_name = ref "str"

let get_filename arg = if (String.contains arg '.') then String.sub arg 0 (String.index arg '.')
						else arg
						
let fwrite name strings =
    let out = open_out name in
      fprintf out "%s\n" strings;
      close_out out

let usage_string =  "Usage: HLHDL COMPILER\n\n" ^ 
					"\t   AST pretty printer:  hlhdl -a <filename>\n" ^
					"\t   Compile to VHDL:  hlhdl -c <filename>\n" ^
					"\t   Test Bench framework:  hlhdl -t <filename>\n"
	  
let _ =
	let action = if (Array.length Sys.argv > 2) then
			try
				List.assoc Sys.argv.(1) [ ("-a", Ast);
										  ("-c", Compile);
										  ("-t", TestBench) ]
			with
				_ -> Error
		else Error in
			let lexbuf = ignore(out_name := get_filename Sys.argv.(2));
						 Lexing.from_channel (open_in Sys.argv.(2)) in
				let program = Parser.program Scanner.token lexbuf in
					match action with
					  (* Pretty printer using AST *)
					  Ast	-> let listing = Ast.string_of_program program in
									fwrite ("./" ^ !out_name ^ ".ast") listing
					| Compile -> 
							let const_listing = (Translate.trans_to_vhdl program) "const"
							and pkg_decl_listing = (Translate.trans_to_vhdl program) "pkg_decl"
							and pkg_body_listing = (Translate.trans_to_vhdl program) "pkg_body"
							and main_listing = (Translate.trans_to_vhdl program) "main" in
								fwrite ("./" ^ !out_name ^ "_defs.vhd") (const_listing ^ pkg_decl_listing ^ pkg_body_listing);
								fwrite ("./" ^ !out_name ^ "_top.vhd") main_listing
					| TestBench -> 
							let tb_frame = (Translate.trans_to_vhdl program) "tbench" in
								fwrite ("./" ^ !out_name ^ "_top_tbframe.vhd") tb_frame					
					| Error -> print_string usage_string
					
					
					
					