----------------------------------------------------
--  COMS W4115 PLT - 2014 Spring Term Project
--              HL-HDL compiler
--             Woon Lee (wgl2107)
----------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use work.globals.all;
use work.lib_comp.all;



ENTITY test6_top is
port (
	clk : in  std_logic;
	rst : in  std_logic;
	csn : in  std_logic;
	oen : in  std_logic;
	wrn : in  std_logic;
	addr : in std_logic_vector(3 downto 0);
	data : inout std_logic_vector(7 downto 0)
);
end test6_top;

architecture RTL of test6_top is
	signal chip_sel :  std_logic;
	signal rd_en :  std_logic;
	signal rd_strb :  std_logic;
	signal rd_strb_latched :  std_logic;
	signal wr_en :  std_logic;
	signal wr_strb :  std_logic;
	signal wr_strb_latched :  std_logic;
	signal addr_latched : std_logic_vector(3 downto 0);
	signal datain : std_logic_vector(7 downto 0);
	signal dataout : std_logic_vector(7 downto 0);
	signal reg_sel_strb : std_logic_vector(15 downto 0);
	signal reg_bank_sel : std_logic_vector(3 downto 0);
begin
	REG0 <= VERSION;
	REG1 <= REVISION;
	chip_sel <= NOT csn;
	rd_en <= NOT oen;
	rd_strb <= chip_sel AND rd_en;
	wr_en <= NOT wrn;
	wr_strb <= chip_sel AND wr_en;
	
	latch_rd : LATCH port map (
		RST => rst,
		EN => '1',
		CLK => clk,
		LAT_IN => rd_strb,
		LAT_OUT => rd_strb_latched 
	);
	
	latch_wr : LATCH port map (
		RST => rst,
		EN => '1',
		CLK => clk,
		LAT_IN => wr_strb,
		LAT_OUT => wr_strb_latched 
	);
	
	latch_addr : LATCH_n_bit
 	generic map(num_bits => 4)
 	port map (
		RST => rst,
		EN => '1',
		CLK => clk,
		LAT_IN => addr,
		LAT_OUT => addr_latched 
	);
	
	latch_datain : LATCH_n_bit
 	generic map(num_bits => 8)
 	port map (
		RST => rst,
		EN => wr_strb,
		CLK => clk,
		LAT_IN => data,
		LAT_OUT => datain 
	);
	inst_cpu_read_reg_read : reg_read port map (
		rst => rst,
		strb_in => rd_strb_latched,
		addr_in => addr_latched,
		data_out => dataout
	);
	data <= dataout;
	four_bank_sel_decode_2_to_4 : decode_2_to_4 port map (
		rst => rst,
		strb_in => wr_strb_latched,
		in_2bit => addr_latched(3 downto 2),
		strb_out => reg_bank_sel
	);
	first_bank_strobes_decode_2_to_4 : decode_2_to_4 port map (
		rst => rst,
		strb_in => reg_bank_sel(0),
		in_2bit => addr_latched(1 downto 0),
		strb_out => reg_sel_strb(3 downto 0)
	);
	second_bank_strobes_decode_2_to_4 : decode_2_to_4 port map (
		rst => rst,
		strb_in => reg_bank_sel(1),
		in_2bit => addr_latched(1 downto 0),
		strb_out => reg_sel_strb(7 downto 4)
	);
	third_bank_strobes_decode_2_to_4 : decode_2_to_4 port map (
		rst => rst,
		strb_in => reg_bank_sel(2),
		in_2bit => addr_latched(1 downto 0),
		strb_out => reg_sel_strb(11 downto 8)
	);
	fourth_bank_strobes_decode_2_to_4 : decode_2_to_4 port map (
		rst => rst,
		strb_in => reg_bank_sel(3),
		in_2bit => addr_latched(1 downto 0),
		strb_out => reg_sel_strb(15 downto 12)
	);
	
	data_write_reg2 : LATCH_n_bit
 	generic map(num_bits => 8)
 	port map (
		RST => rst,
		EN => reg_sel_strb(2),
		CLK => clk,
		LAT_IN => datain,
		LAT_OUT => REG2 
	);
	
	data_write_reg3 : LATCH_n_bit
 	generic map(num_bits => 8)
 	port map (
		RST => rst,
		EN => reg_sel_strb(3),
		CLK => clk,
		LAT_IN => datain,
		LAT_OUT => REG3 
	);
	
	data_write_reg4 : LATCH_n_bit
 	generic map(num_bits => 8)
 	port map (
		RST => rst,
		EN => reg_sel_strb(4),
		CLK => clk,
		LAT_IN => datain,
		LAT_OUT => REG4 
	);
	
	data_write_reg5 : LATCH_n_bit
 	generic map(num_bits => 8)
 	port map (
		RST => rst,
		EN => reg_sel_strb(5),
		CLK => clk,
		LAT_IN => datain,
		LAT_OUT => REG5 
	);
	
	data_write_reg6 : LATCH_n_bit
 	generic map(num_bits => 8)
 	port map (
		RST => rst,
		EN => reg_sel_strb(6),
		CLK => clk,
		LAT_IN => datain,
		LAT_OUT => REG6 
	);
	
	data_write_reg7 : LATCH_n_bit
 	generic map(num_bits => 8)
 	port map (
		RST => rst,
		EN => reg_sel_strb(7),
		CLK => clk,
		LAT_IN => datain,
		LAT_OUT => REG7 
	);
	
	data_write_reg8 : LATCH_n_bit
 	generic map(num_bits => 8)
 	port map (
		RST => rst,
		EN => reg_sel_strb(8),
		CLK => clk,
		LAT_IN => datain,
		LAT_OUT => REG8 
	);
	
	data_write_reg9 : LATCH_n_bit
 	generic map(num_bits => 8)
 	port map (
		RST => rst,
		EN => reg_sel_strb(9),
		CLK => clk,
		LAT_IN => datain,
		LAT_OUT => REG9 
	);
	
	data_write_reg10 : LATCH_n_bit
 	generic map(num_bits => 8)
 	port map (
		RST => rst,
		EN => reg_sel_strb(10),
		CLK => clk,
		LAT_IN => datain,
		LAT_OUT => REG10 
	);
	
	data_write_reg11 : LATCH_n_bit
 	generic map(num_bits => 8)
 	port map (
		RST => rst,
		EN => reg_sel_strb(11),
		CLK => clk,
		LAT_IN => datain,
		LAT_OUT => REG11 
	);
	
	data_write_reg12 : LATCH_n_bit
 	generic map(num_bits => 8)
 	port map (
		RST => rst,
		EN => reg_sel_strb(12),
		CLK => clk,
		LAT_IN => datain,
		LAT_OUT => REG12 
	);
	
	data_write_reg13 : LATCH_n_bit
 	generic map(num_bits => 8)
 	port map (
		RST => rst,
		EN => reg_sel_strb(13),
		CLK => clk,
		LAT_IN => datain,
		LAT_OUT => REG13 
	);
	
	data_write_reg14 : LATCH_n_bit
 	generic map(num_bits => 8)
 	port map (
		RST => rst,
		EN => reg_sel_strb(14),
		CLK => clk,
		LAT_IN => datain,
		LAT_OUT => REG14 
	);
	
	data_write_reg15 : LATCH_n_bit
 	generic map(num_bits => 8)
 	port map (
		RST => rst,
		EN => reg_sel_strb(15),
		CLK => clk,
		LAT_IN => datain,
		LAT_OUT => REG15 
	);
end RTL;


